@@ -4,6 +4,7 @@ import builtins
 from dotenv import load_dotenv, find_dotenv
 import functools
 import sentry_sdk
+from sentry_sdk import utils as sentry_utils
 import os
 
 # set __version__ module
@@ -30,9 +31,11 @@ from toolchest_client.api.exceptions import ToolchestException, DataLimitError,
 from toolchest_client.api.query import Query
 from toolchest_client.api.status import Status, get_status
 from toolchest_client.api.urls import get_api_url, set_api_url
-from .tools.api import alphafold, bowtie2, cellranger_count, clustalo, demucs, diamond_blastp, diamond_blastx, kraken2,\
-    megahit, rapsearch, rapsearch2, shi7, shogun_align, shogun_filter, STAR, test, unicycler
+from .tools.api import add_database, alphafold, bowtie2, cellranger_count, clustalo, demucs, diamond_blastp, \
+    diamond_blastx, kraken2, megahit, rapsearch, rapsearch2, shi7, shogun_align, shogun_filter, STAR, test, unicycler, \
+    update_database
 
+sentry_utils.MAX_STRING_LENGTH = 8192  # monkey patch for Sentry max message length
 sentry_sdk.init(
     None,
 
