@@ -30,26 +30,24 @@ FOUR_POINT_FIVE_GIGABYTES = int(4.5 * 1024 * 1024 * 1024)
 
 
 class Tool:
-    def __init__(self, tool_name, tool_version, tool_args, output_name,
-                 inputs, min_inputs, max_inputs=None, output_path=None,
+    def __init__(self, tool_name, tool_version, tool_args,
+                 inputs, min_inputs=1, max_inputs=100, output_path=None,
                  output_primary_name=None, database_name=None,
                  database_version=None, custom_database_path=None,
                  input_prefix_mapping=None, parallel_enabled=False,
                  max_input_bytes_per_file=FOUR_POINT_FIVE_GIGABYTES,
                  max_input_bytes_per_file_parallel=FOUR_POINT_FIVE_GIGABYTES,
                  group_paired_ends=False, compress_inputs=False,
-                 output_type=OutputType.FLAT_TEXT, output_is_directory=True,
-                 output_names=None, is_async=False, skip_decompression=False):
+                 output_type=OutputType.FLAT_TEXT, expected_output_file_names=None,
+                 is_async=False, is_database_update=False, skip_decompression=False):
         self.tool_name = tool_name
         self.tool_version = tool_version
         self.tool_args = tool_args
-        self.output_name = output_name
-        self.output_primary_name = output_primary_name
         self.output_path = output_path
+        self.output_primary_name = output_primary_name
         if self._output_path_is_local():
             # absolutize path, expand user tilde if present
             self.output_path = os.path.abspath(os.path.expanduser(output_path))
-        self.output_is_directory = output_is_directory
         self.inputs = inputs if isinstance(inputs, str) else list(filter(lambda file: file is not None, inputs))
         # input_prefix_mapping is a dict in the shape of:
         # {
@@ -77,8 +75,9 @@ class Tool:
         self.terminating = False
         self.output_type = output_type or OutputType.FLAT_TEXT
         self.thread_outputs = {}
-        self.output_names = output_names or []
+        self.expected_output_file_names = expected_output_file_names or []
         self.is_async = is_async
+        self.is_database_update = is_database_update
         self.skip_decompression = skip_decompression
         signal.signal(signal.SIGTERM, self._handle_termination)
         signal.signal(signal.SIGINT, self._handle_termination)
@@ -184,7 +183,6 @@ class Tool:
             # Disable parallelization, validation, and revert to plain compressed output
             self.output_validation_enabled = False
             self.parallel_enabled = False
-            self.output_is_directory = True
             self.output_type = OutputType.GZ_TAR
 
         sanitized_args = " ".join(sanitized_args)
@@ -202,8 +200,6 @@ class Tool:
 
         if self.inputs is None:
             raise ValueError("No input provided.")
-        if not self.output_name:
-            raise ValueError("Output name must be non-empty.")
 
     def _merge_outputs(self, output_file_paths):
         """Merges output files for parallel runs."""
@@ -211,9 +207,8 @@ class Tool:
 
     def _warn_if_outputs_exist(self):
         """Warns if default output files already exist in the output directory"""
-        output_dir = self.output_path if self.output_is_directory else os.path.dirname(self.output_path)
-        for file_path in self.output_names + ["output", "output.tar.gz"]:
-            joined_file_path = os.path.join(output_dir, file_path)
+        for file_path in self.expected_output_file_names + ["output", "output.tar.gz"]:
+            joined_file_path = os.path.join(self.output_path, file_path)
             if os.path.exists(joined_file_path):
                 print(f"WARNING: {joined_file_path} already exists and will be overwritten")
 
@@ -225,18 +220,13 @@ class Tool:
         # Check if the given output_path is a directory, if required by the tool
         # and if the user provides output_path.
         if self._output_path_is_local():
-            if self.output_is_directory:
-                if os.path.exists(self.output_path):
-                    if os.path.isfile(self.output_path):
-                        raise ValueError(
-                            f"{self.output_path} is a file. Please pass a directory instead of an output file."
-                        )
-                else:
-                    os.makedirs(self.output_path, exist_ok=True)
-                self._warn_if_outputs_exist()
+            if os.path.exists(self.output_path):
+                if os.path.isfile(self.output_path):
+                    raise ValueError(
+                        f"{self.output_path} is a file. Please pass a directory instead of an output file."
+                    )
             else:
-                os.makedirs(os.path.dirname(self.output_path), exist_ok=True)
-
+                os.makedirs(self.output_path, exist_ok=True)
             self._warn_if_outputs_exist()
 
     def _postflight(self, output):
@@ -244,14 +234,9 @@ class Tool:
         if self._output_path_is_local() and not self.is_async:
             if self.output_validation_enabled:
                 print("Checking output...")
-                if self.output_is_directory:
-                    for output_name in self.output_names:
-                        output_file_path = f"{self.output_path}/{output_name}"
-                        sanity_check(output_file_path)
-                else:
-                    for output_name in self.output_names:
-                        output_file_path = f"{os.path.dirname(self.output_path)}/{output_name}"
-                        sanity_check(output_file_path)
+                for output_file_name in self.expected_output_file_names:
+                    output_file_path = f"{self.output_path}/{output_file_name}"
+                    sanity_check(output_file_path)
 
     def _system_supports_parallel_execution(self):
         """Checks if parallel execution is supported on the platform.
@@ -286,7 +271,7 @@ class Tool:
         # Jupyter notebooks and Windows don't always support the canonical "clear-to-end-of-line" escape sequence
         max_length = 120
         status_message = f"\r{job_count} | {jobs_duration} {status_count}"
-        print(f"{status_message}{(max_length - len(status_message)) * ' '}", end="\r")
+        print(f"{status_message}".ljust(max_length), end="\r")
 
     def _handle_termination(self, signal_number, *_):
         """
@@ -313,7 +298,7 @@ class Tool:
             if thread.is_alive() and thread_status not in {ThreadStatus.COMPLETE, ThreadStatus.FAILED}:
                 self.query_thread_statuses[thread_name] = ThreadStatus.INTERRUPTING
 
-        self._wait_for_threads_to_finish(check_health=False)
+        self._wait_for_threads_to_finish(check_health=False, wait_for_upload=False)
 
     def _check_thread_health(self):
         """Checks for any thread that has ended without reaching a "complete" state, and propagates errors"""
@@ -324,9 +309,13 @@ class Tool:
                 self._kill_query_threads()
                 raise ToolchestException("A job irrecoverably failed. See logs above for details.")
 
-    def _wait_for_threads_to_finish(self, check_health=True):
+    def _wait_for_threads_to_finish(self, check_health=True, wait_for_upload=True):
         """Waits for all jobs and their corresponding threads to finish while printing their statuses."""
         elapsed_seconds = 0
+        # Wait for all threads to finish uploading
+        if wait_for_upload:
+            self._wait_for_threads_to_upload()
+
         for thread in self.query_threads:
             increment_seconds = 5
             while thread.is_alive():
@@ -335,13 +324,34 @@ class Tool:
                 self._pretty_print_pipeline_segment_status(elapsed_seconds)
                 elapsed_seconds += increment_seconds
                 time.sleep(increment_seconds)
-            self._pretty_print_pipeline_segment_status(elapsed_seconds)
+            thread_name = thread.getName()
+            thread_final_status = self.query_thread_statuses.get(thread_name)
+            if thread_final_status == ThreadStatus.COMPLETE:
+                self._pretty_print_pipeline_segment_status(elapsed_seconds)
         print("")
 
         # Double check all threads are complete for safety
         for thread in self.query_threads:
             thread.join()
 
+    def _wait_for_threads_to_upload(self):
+        """Waits for all jobs to finish uploading. To be used only at the start of a run."""
+        while True:
+            statuses = []
+            for thread in self.query_threads:
+                thread_name = thread.getName()
+                statuses.append(self.query_thread_statuses.get(thread_name))
+            uploading = any(
+                map(lambda status: status in [ThreadStatus.INITIALIZING, ThreadStatus.INITIALIZED,
+                                              ThreadStatus.UPLOADING], statuses)
+            )
+            if not uploading:
+                break
+            # Verify that all threads are healthy while uploading
+            self._check_thread_health()
+            time.sleep(5)
+        print("Finished spawning jobs.")
+
     def _generate_jobs(self, should_run_in_parallel):
         """Generates staggered jobs for both parallel and non-parallel runs.
 
@@ -410,8 +420,7 @@ class Tool:
         # Note that this is relying on a result from the generator, so these are slightly staggered
         temp_input_file_paths = []
         temp_output_file_paths = []
-        non_parallel_output_path = f"{self.output_path}/{self.output_name}" if self.output_is_directory \
-            and self._output_path_is_local() else self.output_path
+        non_parallel_output_path = self.output_path
         for thread_index, input_files in enumerate(jobs):
             # Add split files for merging and later deletion, if running in parallel
             # TODO: handle parallel output for s3 uri
@@ -428,20 +437,21 @@ class Tool:
             )
 
             # Deep copy to make thread safe
+            # Note: multithreaded download may be broken with output_path refactor
             query_args = copy.deepcopy({
+                "custom_database_path": self.custom_database_path,
+                "database_name": self.database_name,
+                "database_version": self.database_version,
+                "input_files": input_files,
+                "input_prefix_mapping": self.input_prefix_mapping,
+                "is_database_update": self.is_database_update,
+                "output_path": temp_parallel_output_file_path if should_run_in_parallel else non_parallel_output_path,
+                "output_primary_name": self.output_primary_name,
+                "output_type": self.output_type,
+                "skip_decompression": self.skip_decompression,
                 "tool_name": self.tool_name,
                 "tool_version": self.tool_version,
                 "tool_args": self.tool_args,
-                "database_name": self.database_name,
-                "database_version": self.database_version,
-                "custom_database_path": self.custom_database_path,
-                "output_name": f"{thread_index}_{self.output_name}" if should_run_in_parallel else self.output_name,
-                "output_primary_name": self.output_primary_name,
-                "input_files": input_files,
-                "input_prefix_mapping": self.input_prefix_mapping,
-                "output_path": temp_parallel_output_file_path if should_run_in_parallel else non_parallel_output_path,
-                "output_type": self.output_type,
-                "skip_decompression": self.skip_decompression,
             })
 
             # Add non-distinct dictionary for status updates
@@ -449,12 +459,10 @@ class Tool:
 
             new_thread = Thread(target=q.run_query, kwargs=query_args)
             self.query_threads.append(new_thread)
+            self.query_thread_statuses[new_thread.getName()] = ThreadStatus.INITIALIZING
 
             print(f"Spawning job #{len(self.query_threads)}...")
             new_thread.start()
-            time.sleep(5)
-
-        print("Finished spawning jobs.")
 
         self._wait_for_threads_to_finish()
 
@@ -494,13 +502,13 @@ class Tool:
             if self.is_async:
                 print(
                     f"\nAsync Toolchest initiation is complete! Your run ID is included in the returned object.\n\n"
-                    f"To check the status of this run, call get_status(run_id=\"{run_id}\").\n"
-                    f"Once it's ready to download, call download(run_id=\"{run_id}\", ...) within 7 days\n"
+                    f"To check the status of this run, call toolchest.get_status(run_id=\"{run_id}\").\n"
+                    f"Once it's ready to download, call toolchest.download(run_id=\"{run_id}\", ...) within 7 days\n"
                     )
             else:
                 print(
                     f"\nYour Toolchest run is complete! The run ID and output locations are included in the return.\n\n"
-                    f"If you need to re-download the results, run download(run_id=\"{run_id}\") within 7 days\n"
+                    f"To re-download the results, run toolchest.download(run_id=\"{run_id}\") within 7 days\n"
                     )
 
         # Note: output information is only returned if parallelization is disabled
